from pyg_base import dt, eq, passthru
from pyg_mongo import mongo_table, mongo_reader, mongo_cursor
import pytest
import pandas as pd

def test_clone_cursor():
    reader = mongo_table('test', 'test', mode = 'r')
    assert isinstance(reader, mongo_reader)
    assert mongo_reader(reader) == reader
    cursor = mongo_cursor(reader)
    assert isinstance(cursor, mongo_cursor)


def test_cursor_reader_writer():
    cursor = mongo_table('test', 'test', mode = 'r', writer = False, reader = False)
    assert cursor._writer() == [passthru]
    assert cursor._reader() == [passthru]


def test_mongo_reader_spec_projection():
    reader = mongo_table('test', 'test', mode = 'r')
    assert reader.projection is None    
    assert reader[['a','b']].projection == ['a', 'b']   
    assert not reader.spec 
    assert reader.find(a = 1).spec == {"a": {"$eq": 1}}  
    assert reader() == reader
    assert reader.project() == reader
    assert reader.exc(a = 1).spec == {"a": {"$ne": 1}}
    assert reader.find(a = 1).spec == {"a": {"$eq": 1}}
    

def test_reader_is_no_writer():
    reader = mongo_table('test', 'test', mode = 'r')
    with pytest.raises(AttributeError):
        reader.insert_one(dict(a =1))    
    with pytest.raises(AttributeError):
        reader.insert_many(dict(a =1))    
    with pytest.raises(AttributeError):
        reader.delete_one(dict(a =1))    
    with pytest.raises(AttributeError):
        reader.delete_many(dict(a =1))    
    with pytest.raises(AttributeError):
        reader.set(dict(a =1))    
    with pytest.raises(AttributeError):
        reader.drop(dict(a =1))    

def test_mongo_reader__read():
    t = mongo_table('test', 'test', pk = 'key')
    t.reset.drop()
    df = pd.Series([1,2])
    t.insert_one(dict(key = 1, df = df))
    reader = mongo_table('test', 'test', mode = 'r')
    raw = reader.read(0, passthru)
    assert eq(reader._read(raw)['df'], df) ## now converted to df
    pair = reader.read([0,0])
    assert eq(pair[0]['df'], df) and eq(pair[1]['df'], df)
    assert "Collection(Database(MongoClient(host=['localhost:27017'], document_class=dict, tz_aware=False, connect=True), 'test'), 'test')" in str(reader)
    t.reset.drop()    
    assert "documents count: 0" in str(reader)


def test_mongo_reader_distinct():
    t = mongo_table('test', 'test', pk = 'key')
    t.reset.drop()
    t.insert_one(dict(key = 'a'))
    t.insert_one(dict(key = 1))
    t.insert_one(dict(key = None))
    t.insert_one(dict(key = dt(0)))
    reader = mongo_table('test', 'test', mode = 'r')

    keys = reader.key
    for key in [None, dt(0), 1, 'a']:
        assert key in keys
        
    t.reset.drop()
    t.insert_one(dict(key = 'a'))
    t.insert_one(dict(key = 'd'))
    t.insert_one(dict(key = 'b'))
    t.insert_one(dict(key = 'e'))
    t.insert_one(dict(key = 'c'))
    assert reader.key == ['a', 'b', 'c', 'd', 'e']
    t.reset.drop()

    reader._whatever = 1
    assert reader._whatever == 1



from pyg_mongo import mongo_table
from pyg_base import dictable, Dict, passthru
import pytest

def test_mongo_reader_allows_passthru():
    c = mongo_table('test', 'test')
    assert c._reader(False) == [passthru]    
    assert c._reader(passthru) == [passthru]    

def test_mongo_cursor():
    c = mongo_table('test', 'test')
    c.drop()    
    docs = dictable(a = range(10)) * dict(b = range(10))
    c = c.insert_many(docs)
    assert len(c) == 100
    assert len(c.a) == 10
    assert len(c.b) == 10
    c = c.set(ab = lambda a, b: a*b)
    assert len(c.ab) == 37
    assert len(c.find(ab = 12)) == 4 ## 4 * 3 and 6 * 2
    assert len(c.find_one(a = 0, b = 0)) == 1
    c = c.delete_one(a = 0, b = 0)
    with pytest.raises(ValueError):
        c.find_one(a = 0, b = 0)
    
    doc = c.find_one(a = 1, b = 1)[0]
    doc['sum'] = 2
    c.update_one(doc)
    new_doc = c.find_one(a = 1, b = 1)[0]
    assert new_doc.sum == 2
    
    doc = dict(status = 'test')
    c = c.update_many(doc)
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test'
    del c['status']
    doc = c.find_one(a = 5, b = 2)[0]
    assert 'status' not in doc  
    
    c['status'] = 'test2'
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test2'


    c = c.set(status = 'test3')
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test3'

    c = c.set(status = 'test4', sum = lambda a,b:a+b)
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test4'
    assert doc.sum == 7    
    
    doc.status = 'test5' ## unlike normal insert, we overwrite since _id in doc
    c.insert_one(doc)
    assert len(c) == 99
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test5'

    del doc['_id']
    doc.status = 'test6: no id so new doc'    
    c.insert_one(doc)
    assert len(c.find(a = 5, b = 2)) == 2
    c.inc(status = 'test6: no id so new doc').drop()
    assert len(c) == 99
    doc = c.find_one(a = 5, b = 2)[0]
    assert doc.status == 'test5'
    
    t = c[::]
    t = t(power = lambda a, b: (a+1)**(b-1))
    ### ids exist so should update
    c.insert_many(t)
    assert len(c) == 99
    doc = c.inc(power = 5)[0]
    assert doc.a == 4 and doc.b == 2
    
    t = t - '_id' ## now we add new docs
    c.insert_many(t)
    assert len(c) == 99 + len(t)
    c = c.drop()
    c = c + t
    assert len(c) == len(t)    

    assert Dict(c.sort('a', 'b')[['a','b']][0]) - '_id' == Dict(a = 0, b = 1)
    del c.sort('a', 'b')[0]

    with pytest.raises(ValueError):
        c.find_one(a = 0, b = 1)

    del c['sum']
    assert 'sum' not in c[0]

    del c[dict(a = 1, b = 2)]
    with pytest.raises(ValueError):
        c.find_one(a = 1, b = 2)

    with pytest.raises(ValueError):
        del c[dict(a = 1, b = 2)]
        
    c + dict(x = 1, y = 2)
    assert c.x == [1]
    c.drop()


def test_cursor_bits():
    c = mongo_table('test', 'test')
    c =  c.drop()
    d = dictable(a = [1,2,3]) * dict(b = [1,2,3])
    c = c.insert_many(d)
    assert c.docs() == dictable(doc = list(c))
    assert dictable(c) == dictable(c.docs().doc)
    assert sorted(c.keys()) == ['_id', 'a', 'b']
    assert sorted(c(reader = False).keys()) == ['_id', '_obj', 'a', 'b']




# -*- coding: utf-8 -*-


from pyg_cell._cell import cell, updated_cell, cell_go, cell_item, cell_func, cell_load, cell_output, cell_outputs, cell_clear, cell_inputs
from pyg_cell._dag import get_DAG, get_DAGs, get_GADs, get_GAD, add_edge, topological_sort, descendants, del_edge
from pyg_cell._acell import acell, acell_load, acell_func, acell_go
from pyg_cell._db_cell import db_cell, db_cell_func, db_load, db_save, get_cell, get_data, load_cell, load_data, cell_push, cell_pull, calculating_cell
from pyg_cell._periodic_cell import periodic_cell
from pyg_cell._latest_cell  import latest_cell
from pyg_cell._cache import db_cache, cell_cache, cell_cache_, db_cache_
from pyg_cell._tree import cell_tree
from pyg_cell._cell_runner import cell_runner
